/* Written by Vincent Autefage */

#include "dnet.h"
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <sys/socket.h>
#include <fcntl.h>

static char *device_addr(int sock, const char *inf)
{
  if(inf == NULL)
    return NULL;
  struct ifreq sinf;
  memset(&sinf, 0, sizeof(sinf));
  strncpy(sinf.ifr_ifrn.ifrn_name, inf, strlen(inf)*sizeof(char));
  if(ioctl(sock, SIOCGIFADDR, &sinf) < 0)
    return NULL;
  struct sockaddr_in *ssinf = (struct sockaddr_in *)&(sinf.ifr_ifru.ifru_addr);
  return inet_ntoa(ssinf->sin_addr);
  
}

int open_tcp_server(int port, unsigned int size_queue, const char *inf)
{
  struct sockaddr_in server;
  int sock = socket(AF_INET, SOCK_STREAM, 0);
  if(sock < 0)
    return DNET_ERR_SOCK;
  
  char * inf_addr = device_addr(sock, inf);
  if(inf != NULL && inf_addr == NULL)
    return DNET_ERR_BINDDEV;

  if (inf_addr != NULL)
    server.sin_addr.s_addr = inet_addr(inf_addr);
  else
    server.sin_addr.s_addr = htonl(INADDR_ANY);

  server.sin_family = AF_INET;
  server.sin_port = htons(port);
 
  if(bind(sock, (struct sockaddr*)&server, sizeof(server)) == -1) 
    return DNET_ERR_BIND;

  size_t length = sizeof(server);
  if (getsockname(sock, (struct sockaddr*)&server, (socklen_t *)&length)) 
    return DNET_ERR_GETSOCKNAME;
  
  return (listen(sock, size_queue) == 0) ? sock : DNET_ERR_LISTEN;
}

int wait_tcp_connection(int sock, char **ip,  int *port)
{
  struct sockaddr addr;
  socklen_t addrlen = sizeof(addr);
  
  int desc = accept(sock, &addr, &addrlen);
  if(desc == -1)
    return DNET_ERR_ACCEPT;

  struct sockaddr_in *peer = (struct sockaddr_in *)&addr;
  *ip = inet_ntoa(peer->sin_addr);  
  *port = peer->sin_port;

  return desc;
}


int open_tcp_client(int port, char *ip, unsigned int retry)
{
  int sock;
  struct sockaddr_in server;

  sock = socket(AF_INET, SOCK_STREAM, 0);
  if (sock < 0) 
    return DNET_ERR_SOCK;

  server.sin_family = AF_INET;
  inet_aton(ip, &server.sin_addr);
  server.sin_port = htons(port);

  while(connect(sock, (struct sockaddr*)&server, sizeof(server)) == -1)
  {
    if(retry > 0)
      retry--;
    else
      return DNET_ERR_CONNECT;
  }
  return sock;
}

int open_udp_server(int port, const char *inf)
{
  struct sockaddr_in server;
  int sock = socket(AF_INET, SOCK_DGRAM, 0);
  if(sock < 0)
    return DNET_ERR_SOCK;
 
  char *inf_addr = device_addr(sock, inf);
  if(inf != NULL && inf_addr == NULL)
    return DNET_ERR_BINDDEV;

  if (inf_addr != NULL)
    server.sin_addr.s_addr = inet_addr(inf_addr);
  else
    server.sin_addr.s_addr = htonl(INADDR_ANY);
 
  server.sin_family = AF_INET;
  server.sin_port = htons(port);
 
  if(bind(sock, (struct sockaddr*)&server, sizeof(server)) == -1) 
    return DNET_ERR_BIND;

  size_t length = sizeof(server);
  if (getsockname(sock, (struct sockaddr*)&server, (socklen_t *)&length)) 
    return DNET_ERR_GETSOCKNAME;

  return sock;
}

int open_udp_client(int port, char *ip)
{
  int sock;
  struct sockaddr_in server;

  sock = socket(AF_INET, SOCK_DGRAM, 0);
  if (sock < 0) 
    return DNET_ERR_SOCK;

  server.sin_family = AF_INET;
  inet_aton(ip, &server.sin_addr);
  server.sin_port = htons(port);
  return sock;
}

int set_nonblock_socket(int sock)
{
  int stat = fcntl(sock, F_GETFL);
  return fcntl(sock, F_SETFL, stat|O_NONBLOCK);
}

int close_tcp_connection(int sock)
{
  shutdown(sock, SHUT_RDWR);
  return close(sock);
}

int close_udp_connection(int sock)
{
  return close(sock);
}


char *gethostip(const char *hostname)
{
  struct hostent *h = gethostbyname(hostname);
  if(h == NULL)
    return NULL;

  struct in_addr *haddr = (struct in_addr *)h->h_addr;
  char *__ip = inet_ntoa(*haddr);

  return __ip;
}
