/* Written by Vincent Autefage */

#ifndef DNET_H
#define DNET_H

#define _GNU_SOURCE

#define DNET_ERR_SOCK -1
#define DNET_ERR_BIND -2
#define DNET_ERR_GETSOCKNAME -3
#define DNET_ERR_BINDDEV -4
#define DNET_ERR_ACCEPT -5
#define DNET_ERR_CONNECT -6
#define DNET_ERR_LISTEN -7

int open_tcp_server(int port, unsigned int size_queue, const char *inf);
/*
  Creates a new tcp server (AF_INET, SOCK_STREAM) on port 'port' 
  with 'size_queue' max connections operating at the same time, 
  and finally returns the corresponding server socket. 
  You can link the server to the interface 'inf'.
  Returns :
    > 0 : Great.
    DNET_ERR_SOCK  : socket() failure.
    DNET_ERR_BIND  : bind() failure.
    DNET_ERR_GETSOCKNAME  : getsockname() failure.
    DNET_ERR_BINDDEV : ioctl() failure [bad inf].
*/

int wait_tcp_connection(int sock, char **ip, int *port);
/*
  Waits a new tcp connection by a client on the server socket 'sock'.
  The client's address is stored in addr and addr's length in addrlen.
  Returns :
    > 0 : The connection socket.
    DNET_ERR_ACCEPT  : accept() failure.
*/

int open_tcp_client(int port, char *ip, unsigned int retry);
/*
  Creates a new client tcp connection (AF_INET, SOCK_STREAM)
  to the server 'ip':'port' and returns the corresponding socket.
  The connection will be retried 'retry' times in case of failure.
  Returns :
    > 0 : Great.
    DNET_ERR_SOCK  : socket() failure.
    DNET_ERR_CONNECT  : connect() failure.
*/

int open_udp_server(int port, const char *inf);
/*
  Creates a new udp server (AF_INET, SOCK_DGRAM) on port 'port'  
  and finally returns the corresponding server socket.
  You can link the server to the interface 'inf'.
  Returns :
    > 0 : Great.
    DNET_ERR_SOCK  : socket() failure.
    DNET_ERR_BIND  : bind() failure.
    DNET_ERR_BINDDEV : setsockopt() failure [bad inf].
*/

int open_udp_client(int port, char *ip);
/*
  Creates a new client udp connection (AF_INET, SOCK_STREAM)
  to the server 'ip':'port' and returns the corresponding socket.
  Returns :
    > 0 : Great.
    DNET_ERR_SOCK  : socket() failure.
 */

int set_nonblock_socket(int sock);
/*
  Set the socket to nonblock.
  Returns :
    0  : Great.
    -1 : Bad.
*/

int close_tcp_connection(int sock);
/*
  Closes the TCP connection stream.
  Returns :
    0  : Great.
    -1 : Bad.
*/

int close_udp_connection(int sock);
/*
  Closes the UDP connection stream.
  Returns :
    0  : Great.
    -1 : Bad.
*/


char *gethostip(const char *hostname);
/*
  Returns the hostname's IP (allocated by malloc).
  Returns :
      0x... : Great.
      NULL  : gethostbyname() failure.
*/

#endif

